@@ -125,7 +125,7 @@ gth_file_source_vfs_get_entry_points (GthFileSource *file_source)
 
 	list = NULL;
 
-	list = gth_file_source_vfs_add_uri (list, file_source, get_home_uri ());
+	list = gth_file_source_vfs_add_uri (list, file_source, _g_uri_get_home ());
 	list = gth_file_source_vfs_add_special_dir (list, file_source, G_USER_DIRECTORY_PICTURES);
 	list = gth_file_source_vfs_add_special_dir (list, file_source, G_USER_DIRECTORY_VIDEOS);
 	list = gth_file_source_vfs_add_special_dir (list, file_source, G_USER_DIRECTORY_DOWNLOAD);
@@ -151,12 +151,6 @@ gth_file_source_vfs_get_entry_points (GthFileSource *file_source)
 			continue;
 		}
 
-		info = g_file_query_info (file, GFILE_BASIC_ATTRIBUTES ",access::*", G_FILE_QUERY_INFO_NONE, NULL, NULL);
-		if (info == NULL) {
-			g_object_unref (file);
-			continue;
-		}
-
 		icon = g_mount_get_symbolic_icon (mount);
 		name = g_mount_get_name (mount);
 
@@ -174,8 +168,15 @@ gth_file_source_vfs_get_entry_points (GthFileSource *file_source)
 			g_free (drive_name);
 		}
 
+		info = g_file_info_new ();
+		g_file_info_set_file_type (info, G_FILE_TYPE_DIRECTORY);
+		g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_READ, TRUE);
+		g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE, FALSE);
+		g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_DELETE, FALSE);
+		g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_TRASH, FALSE);
 		g_file_info_set_symbolic_icon (info, icon);
 		g_file_info_set_display_name (info, name);
+		g_file_info_set_name (info, name);
 
 		list = g_list_append (list, gth_file_data_new (file, info));
 		g_object_unref (info);
@@ -193,14 +194,7 @@ static GFile *
 gth_file_source_vfs_to_gio_file (GthFileSource *file_source,
 				 GFile         *file)
 {
-	char  *uri;
-	GFile *gio_file;
-
-	uri = g_file_get_uri (file);
-	gio_file = g_file_new_for_uri (g_str_has_prefix (uri, "vfs+") ? uri + 4 : uri);
-	g_free (uri);
-
-	return gio_file;
+	return g_file_dup (file);
 }
 
 
@@ -209,18 +203,16 @@ gth_file_source_vfs_get_file_info (GthFileSource *file_source,
 				   GFile         *file,
 				   const char    *attributes)
 {
-	GFile     *gio_file;
 	GFileInfo *file_info;
 	char      *uri;
 
-	gio_file = gth_file_source_to_gio_file (file_source, file);
-	file_info = g_file_query_info (gio_file,
+	file_info = g_file_query_info (file,
 				       attributes,
 				       G_FILE_QUERY_INFO_NONE,
 				       NULL,
 				       NULL);
 
-	uri = g_file_get_uri (gio_file);
+	uri = g_file_get_uri (file);
 	if (g_strcmp0 (uri, "file:///") == 0) {
 		GIcon *icon;
 
@@ -230,11 +222,10 @@ gth_file_source_vfs_get_file_info (GthFileSource *file_source,
 
 		g_object_unref (icon);
 	}
-	else if (g_strcmp0 (uri, get_home_uri ()) == 0)
+	else if (g_strcmp0 (uri, _g_uri_get_home ()) == 0)
 		g_file_info_set_display_name (file_info, _("Home Folder"));
 
 	g_free (uri);
-	g_object_unref (gio_file);
 
 	return file_info;
 }
@@ -321,7 +312,6 @@ gth_file_source_vfs_for_each_child (GthFileSource        *file_source,
 				    gpointer              user_data)
 {
 	GthFileSourceVfs *file_source_vfs = (GthFileSourceVfs *) file_source;
-	GFile            *gio_folder;
 
 	gth_file_source_set_active (file_source, TRUE);
 	g_cancellable_reset (gth_file_source_get_cancellable (file_source));
@@ -335,8 +325,7 @@ gth_file_source_vfs_for_each_child (GthFileSource        *file_source,
 	file_source_vfs->priv->user_data = user_data;
 	file_source_vfs->priv->check_hidden_files = _g_file_attributes_matches_any (attributes, G_FILE_ATTRIBUTE_STANDARD_IS_HIDDEN);
 
-	gio_folder = gth_file_source_to_gio_file (file_source, parent);
-	g_directory_foreach_child (gio_folder,
+	_g_directory_foreach_child (parent,
 				   recursive,
 				   TRUE,
 				   attributes,
@@ -345,8 +334,6 @@ gth_file_source_vfs_for_each_child (GthFileSource        *file_source,
 				   fec__for_each_file_func,
 				   fec__done_func,
 				   file_source);
-
-	g_object_unref (gio_folder);
 }
 
 
@@ -391,25 +378,24 @@ gth_file_source_vfs_copy (GthFileSource    *file_source,
 	cod->ready_callback = ready_callback;
 	cod->user_data = data;
 
-	_g_copy_files_async (file_list,
-			     destination->file,
-			     move,
-			     GTH_FILE_COPY_ALL_METADATA | GTH_FILE_COPY_RENAME_SAME_FILE,
-			     GTH_OVERWRITE_RESPONSE_UNSPECIFIED,
-			     G_PRIORITY_DEFAULT,
-			     gth_file_source_get_cancellable (file_source),
-			     progress_callback,
-			     data,
-			     dialog_callback,
-			     data,
-			     copy_done_cb,
-			     cod);
+	_g_file_list_copy_async (file_list,
+				 destination->file,
+				 move,
+				 GTH_FILE_COPY_ALL_METADATA | GTH_FILE_COPY_RENAME_SAME_FILE,
+				 GTH_OVERWRITE_RESPONSE_UNSPECIFIED,
+				 G_PRIORITY_DEFAULT,
+				 gth_file_source_get_cancellable (file_source),
+				 progress_callback,
+				 data,
+				 dialog_callback,
+				 data,
+				 copy_done_cb,
+				 cod);
 }
 
 
 static gboolean
-gth_file_source_vfs_can_cut (GthFileSource *file_source,
-			     GFile         *file)
+gth_file_source_vfs_can_cut (GthFileSource *file_source)
 {
 	return TRUE;
 }
@@ -884,6 +870,37 @@ gth_file_source_vfs_remove (GthFileSource *file_source,
 }
 
 
+static GdkDragAction
+gth_file_source_vfs_get_drop_actions (GthFileSource *file_source,
+				      GFile         *destination,
+				      GFile         *file)
+{
+	GdkDragAction  actions = 0;
+	char          *dest_uri;
+	char          *file_uri;
+	char          *dest_scheme;
+	char          *file_scheme;
+
+	dest_uri = g_file_get_uri (destination);
+	dest_scheme = gth_main_get_source_scheme (dest_uri);
+	file_uri = g_file_get_uri (file);
+	file_scheme = gth_main_get_source_scheme (file_uri);
+
+	if (_g_str_equal (dest_scheme, "file")
+		&& _g_str_equal (file_scheme, "file"))
+	{
+		actions = GDK_ACTION_COPY | GDK_ACTION_MOVE;
+	}
+
+	g_free (file_scheme);
+	g_free (file_uri);
+	g_free (dest_scheme);
+	g_free (dest_uri);
+
+	return actions;
+}
+
+
 static void
 gth_file_source_vfs_finalize (GObject *object)
 {
@@ -927,6 +944,7 @@ gth_file_source_vfs_class_init (GthFileSourceVfsClass *class)
 	file_source_class->monitor_entry_points = gth_file_source_vfs_monitor_entry_points;
 	file_source_class->monitor_directory = gth_file_source_vfs_monitor_directory;
 	file_source_class->remove = gth_file_source_vfs_remove;
+	file_source_class->get_drop_actions = gth_file_source_vfs_get_drop_actions;
 }
 
 
@@ -950,5 +968,5 @@ gth_file_source_vfs_init (GthFileSourceVfs *file_source)
 	file_source->priv->mount_monitor = NULL;
 	file_source->priv->check_hidden_files = FALSE;
 
-	gth_file_source_add_scheme (GTH_FILE_SOURCE (file_source), "vfs+");
+	gth_file_source_add_scheme (GTH_FILE_SOURCE (file_source), "file");
 }
